/*
 * elfspe - A wrapper to allow direct execution of SPE binaries
 * Copyright (C) 2005 IBM Corp.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License,
 * or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
 *  License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software Foundation,
 *   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>

#include <spe.h>

static void *spe = NULL;

static void *
mmap_file (const char *filename, off_t size, int rw)
{
  int flags = rw? O_RDWR : O_RDONLY;
  int prot = rw? (PROT_READ | PROT_WRITE) : PROT_READ;
  size_t ps = getpagesize ();

  void *addr;
  int fd;

  if ((fd = open (filename, flags)) < 0)
    return NULL;

  addr = mmap (NULL, (size + ps - 1) & ~(ps - 1), prot, MAP_SHARED, fd, 0);
  if (addr == MAP_FAILED)
    return NULL;

  close (fd);

  return addr;
}

static void handler (int signr ) __attribute__ ((noreturn));
static void
handler (int signr)
{
  if (spe)
    spe_cleanup (spe);

  fprintf (stderr, "Killed by signal %d\n", signr);
  exit (128 + signr);
}

int
main (int argc, char **argv)
{
  spe_gid_t gid;
  struct stat st;
  void *file;
  int rc;

  signal (SIGSEGV, handler);
  signal (SIGBUS, handler);
  signal (SIGILL, handler);
  signal (SIGFPE, handler);
  signal (SIGTERM, handler);
  signal (SIGHUP, handler);
  signal (SIGINT, handler);

  if (argc != 2)
    {
      fprintf (stderr, "Usage: elfspe [spe-elf]\n");
      exit (1);
    }

  if (stat (argv[1], &st) < 0)
    {
      perror (argv[1]);
      exit (1);
    }

  file = mmap_file (argv[1], st.st_size, 0);
  if (!file)
    {
      perror (argv[1]);
      exit (1);
    }

  gid = spe_create_group(0,0,0,0);
  if (!gid)
    {
      perror ("create_group");
      exit (1);
    }

  
  spe = spe_setup (gid, file, NULL, NULL, SPE_PTRACE_ME);
  if (!spe)
    {
      perror ("spe_setup");
      exit (1);
    }

  rc = do_spe_run (spe);
  spe_cleanup (spe);

  return rc & 0xff;
}

