/*
 * libspe - A wrapper library to adapt the JSRE SPU usage model to SPUFS 
 * Copyright (C) 2005 IBM Corp. 
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by 
 * the Free Software Foundation; either version 2.1 of the License, 
 * or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful, but 
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 *  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 *  License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License 
 *   along with this library; if not, write to the Free Software Foundation, 
 *   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _spe_h_
#define _spe_h_

typedef void *speid_t;
typedef void *spe_gid_t;

/* Flags for spe_create_thread
 */

#define SPE_PTRACE_ME		  0x0001	/* Allow debugger to attach   */

/* mem_access
 */
#define SPE_MEM_PRIVATE         0
#define SPE_MEM_SHARED          1

/* policy
 */
#define SPE_SCHED_OTHER         0
#define SPE_SCHED_FIFO          1
#define SPE_SCHED_RR            2


/* spe_ctx: A SPE context.
 */
struct spe_ctx {
        unsigned int    npc;
        unsigned int    status;
        unsigned long   reg[128][4];
        void            *ls;
};

/* Low-level SPE execution API.
 */
extern void *spe_gid_setup(int mem_access, int policy, int priority, int use_events);
extern void *spe_setup (spe_gid_t gid, void *start, void *argp, void *envp, int flags);
extern void spe_cleanup (void *spe);
extern int do_spe_run (void *spe);
extern unsigned int set_npc (void *spe, unsigned int npc);

extern void *load_binary(char *bin);

extern int spe_mbox_read(speid_t speid, unsigned int *val);
extern int spe_ibox_read(speid_t speid, unsigned int *val);
extern int spe_wbox_write(speid_t speid, unsigned int val);


/* APIs for SPE threads.
 */

extern spe_gid_t spe_create_group (int mem_access, int policy, int priority, int use_events);
extern int spe_group_max (spe_gid_t spe_gid);

extern speid_t spe_create_thread (spe_gid_t gid, void *start,
				  void *argp, void *envp,
				  int mask, int flags);

extern int spe_wait (speid_t speid, int *status, int options);
extern int spe_wait_save (speid_t speid, struct spe_ctx *spe_context);

extern int spe_kill (speid_t speid, int sig);

extern void *spe_get_ls(speid_t speid);
extern void *spe_get_ps(speid_t speid);

#endif
